require('mb-defines')
require('mb-functions')

function is_value_in_list (value, list)
  for i, v in pairs (list) do
    if value == v then
      return true
    end
  end
  return false
end

-- function lists_intersection (list1, list2)
  -- local new_list = {}
  -- for i, v in pairs (list1) do
    -- for j, w in pairs (list2) do
      -- if v == w and not is_value_in_list (v, new_list) then
        -- table.insert (new_list, v)
      -- end
    -- end
  -- end
-- end

function get_resource_items_list (resource_type)
  local resource_type = resource_type or 'item'
  local items_or_fluids_list = {}
  
  local autoplace_entities_list = {}
  for entity_name, autoplace_prototype in pairs (game.autoplace_control_prototypes) do
    table.insert (autoplace_entities_list, entity_name)
  end
  log ('autoplace_entities_list: '..serpent.line (autoplace_entities_list))
  
  -- for entity_name, entity_prototype in pairs (game.entity_prototypes) do
  for i, entity_name in pairs (autoplace_entities_list) do
    local entity_prototype = game.entity_prototypes[entity_name]    
    if entity_prototype and entity_prototype.mineable_properties and entity_prototype.type == 'resource' then
      -- log (entity_name..' has mineable_properties: '.. serpent.line(entity_prototype.mineable_properties))
      if entity_prototype.mineable_properties.products then
        for j, product in pairs (entity_prototype.mineable_properties.products) do
          if product.type == resource_type and not is_value_in_list (product.name, items_or_fluids_list) then
            table.insert (items_or_fluids_list, product.name)
          end
        end
        -- log (entity_name..' products: '.. serpent.line(entity_prototype.mineable_properties.products))
        log (entity_name..' ' .. entity_prototype.type..' products: '.. serpent.line(entity_prototype.mineable_properties))
      end
    end
  end
  
  log ('type: '..resource_type..', items_or_fluids_list: '..serpent.line (items_or_fluids_list))
  
  return items_or_fluids_list
  
end

script.on_load(on_load);

script.on_init(function()
  -- local resource_items = {'iron-ore', 'copper-ore', 'stone', 'coal'}
  local resource_items = get_resource_items_list ('item')
  local resource_fluids = get_resource_items_list ('fluid')
  
  

  if not (global) then global = {} end
  if not (global.resource_items) then global.resource_items = resource_items end
  if not (global.resource_fluids) then global.resource_fluids = resource_fluids end
  
  local force = game.forces.player
  if not (global.force) then global.force = force end
  force.research_all_technologies()
  force.manual_crafting_speed_modifier = -1
  force.character_logistic_slot_count = 0
  force.character_trash_slot_count = 0
  force.disable_research()

  game.disable_tips_and_tricks();
  
  local surface = game.surfaces.nauvis
  if not (global.surface) then global.surface = surface end
  
  local x = -31.5
  local y = -28.5
  
  if not (global.belts) then global.belts = {} end
  for i, item in pairs (resource_items) do
    -- local name = item
    local item_stack = {name=item, count=1}
    local position = {x = x, y = y}
    local entity = surface.create_entity
      {
        name = "express-underground-belt", 
        position = position,
        direction = 2, -- east
        force = force,
        type = "output"
      }
    protect_entity(entity)
    add_item_to_belt(entity, item_stack)
    table.insert (global.belts, {item = item, item_stack = item_stack, entity = entity})
    y = y + 2
  end
  
  if not (global.pipes) then global.pipes = {} end
  for i, fluid in pairs (resource_fluids) do
    -- local name = fluid
    -- local item_stack = {name=item, count=1}
    local position = {x = x, y = y}
    local entity = surface.create_entity
      {
        name = "pipe-to-ground", 
        position = position, 
        direction = 2, -- east
        force = force,
        -- type = "output"
      }
    protect_entity(entity)
    add_fluid_to_pipe(entity, fluid)
    table.insert (global.pipes, {fluid = fluid, entity = entity})
    y = y + 2
  end
  
  

  -- global.iron_belt = surface.find_entity("express-underground-belt",{-33.5,-13.5});
  -- global.copper_belt = surface.find_entity("express-underground-belt",{-33.5,-4.5});
  -- global.coal_belt = surface.find_entity("express-underground-belt",{-33.5,4.5});
  -- global.oil_pipe = surface.find_entity("pipe-to-ground",{-33.5,13.5});
  global.forbidden_items = {["iron-ore"]=true,["copper-ore"]=true,["coal"]=true};
  global.characters = {};
  global.running = false;
  surface.always_day=true;
  on_load();
  -- protect_entity(iron_belt);
  -- protect_entity(copper_belt);
  -- protect_entity(coal_belt);
  -- protect_entity(oil_pipe);

  local recipes = force.recipes;
  for name,recipe in pairs(recipes) do
    recipe.enabled = false;
  end
  
  for k,name in ipairs(craftable_recipes) do
    local recipe = recipes[name];
    recipes[name].enabled = true;
    for k,product in pairs(recipe.products) do
      if product.type == "item" then
        forbidden_items[product.name] = true;
      end
    end
  end
end)



script.on_event(defines.events.on_player_main_inventory_changed, remove_forbidden_items);

script.on_event(defines.events.on_player_quickbar_inventory_changed, remove_forbidden_items);

script.on_event(defines.events.on_tick, function(event)
  local tick = event.tick;
  
  if tick % 3 == 0 then -- 20 operations/second, 40 items/second, 2400 items/min
    local belts = global.belts
    for i, belt in pairs (belts) do
      add_item_to_belt(belt.entity, belt.item_stack);
    end
  end
  
  -- if tick % 10 == 0 then
    local pipes = global.pipes
    for i, pipe in pairs (pipes) do
      add_fluid_to_pipe(pipe.entity, pipe.fluid);
    end
  -- end

  
  if global.running and global.finished == nil then
    if global.run_start_time % 60 == game.tick % 60 then
      update_status();
    end;
    if repair_combinators and game.tick > global.run_start_time then
      for control, value in pairs(repair_combinators) do
        if control.valid then control.parameters = value end;
      end
      repair_combinators = nil;
    end
  end
end)

script.on_event(defines.events.on_player_created, function(event)
  local player = game.players[event.player_index]
  player.minimap_enabled = false;
  player.disable_recipe_groups();
  player.game_view_settings.show_research_info = false;
  setup_player_base(player)
  if global.player_setup == nil then
    setup_player_inventory(player)
    global.player_setup = true
  end;
end)

script.on_event(defines.events.on_player_joined_game, function(event)
  local player = game.players[event.player_index]
  if player.character then
    global.characters[player.index] = player.character;
  end
  if global.running then
    create_validating_ui(player);
    player.set_controller{type=defines.controllers.ghost }
  else
    set_player_character(player);
    create_planning_ui(player);
  end
end);

script.on_event(defines.events.on_gui_click, function(event)
  local element = event.element;
  local handler = button_clicks[element.name];
  if handler ~= nil then
    handler(event.player_index);
  end
end)

script.on_event(defines.events.on_rocket_launched, function(event)
  if not global.running or not global.rocket_launched then return end;
  global.finished = true;
  if not global.won then
    game.set_game_state{game_finished = true, player_won = true, can_continue = true }
    global.won = true;
  end
  for _,player in pairs(game.connected_players) do
    local container = player.gui.center;
    local frame = container.add{type="frame", direction="vertical", name="win_panel" }
    -- frame.add{type="label",caption="Congratulations!", name="header", style="frame_caption_label_style" }
    frame.add{type="label",caption="Congratulations!", name="header", style="frame_caption_label" }
    -- frame.add{type="label",caption="Your time is "..get_time_passed(), name="time", style="frame_caption_label_style" }
    frame.add{type="label",caption="Your time is "..get_time_passed(), name="time", style="frame_caption_label" }
    frame.add{type="button",caption="Close", name="close_win_panel"}
  end
end)